@@ -4,6 +4,8 @@
 
  ## evaluate single reduct
  res.1 <- FS.quickreduct.RST(decision.table)
+ print(res.1)
  
  ## generate new decision table according to the reduct
- new.decTable <- SF.applyDecTable(decision.table, res.1)
\ No newline at end of file
+ new.decTable <- SF.applyDecTable(decision.table, res.1)
+ print(new.decTable)
\ No newline at end of file
